#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: factory_defaults.pl                                               #
#                                                                             #
# Description: Script to save the current files and restore the default       #
#              password files.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

#use the shell directly
use Shell qw(cp rm mv);

#variables to make life easier
$BackupDirectory = "/usr/ins/intel/factdef";       #default directory for backups
$RestoreDirectory = "/etc";                        #directory to restore password files
$PasswordDirectory = "/etc";                       #directory to find password files to backup
$Backup = 0;                                       #backup flag
$Restore = 0;                                      #restore flag
$MAX_BACKUPS = 5;                                  #basically a constant to signify the maximum backups kept

#get the paramaters r(Restore) b(Backup) t(test)
foreach $i (0 .. $#ARGV)
{
   if (@ARGV[$i] =~ /-/)                    # look for the '-' specifying this is a paramater
   {
      if (@ARGV[$i] =~ /b/)                 # b means that we need to backup the current files
      {
         $Backup = 1;
      }
      if (!$Restore && @ARGV[$i] =~ /r/)                 # r means that we are restoring the default files
      {
         if (@ARGV[$i] =~ /r=/)             # there is a particular restore requested
         {
            if (@ARGV[$i] =~ /r=default/ || @ARGV[$i] =~ /r=first/ || @ARGV[$i] =~ /r=0/)   # default means we want the factory defaults
            {
               $Restore = -1;
            }
            else
            {
               # find out what the last valid backup set is
               for ($LastBackup = $MAX_BACKUPS; $LastBackup > 0 && !(-e "$BackupDirectory/passwd.~$LastBackup~"); --$LastBackup)
               {
               }      
               print ("Last Backup set is $LastBackup\n");

               for ($Restore = $MAX_BACKUPS; $Restore > 0  && !((@ARGV[$i] =~ /r=$Restore/ || @ARGV[$i] =~ /r=last/) && $LastBackup >= $Restore); --$Restore)
               {
               }
               print ("Restoration set is $Restore\n");
               
               if (!$Restore)
               {
                  die "Invalid option \"@ARGV[$i]\". $LastBackup of $MAX_BACKUPS sets currently exist.  Format is -r[=1...$MAX_BACKUPS,default,first,last]. No actions were performed.  Stopped";
               }
            }
         }
         else
         {
            $Restore = -1;                   # just r means restore the defaults
         }
      }
      if (@ARGV[$i] =~ /t/)                 # t means that we are in test mode and won't hurt ourselves
      {
         $RestoreDirectory = $RestoreDirectory . "/temp";
         print ("\nWe are in test mode, restoration will be to $RestoreDirectory\n");
         `mkdir -p --mode=700 $RestoreDirectory`;  #for testing and debugging only
      }
   }
}


if ($Backup)
{
   print ("Backing up to the $BackupDirectory\n");

   #in case the backup folder doesn't yet exist
   `mkdir -p --mode=700 $BackupDirectory`;

   #backup the old files to a numbered backup just in case someone wants to look at them later
   if (chdir $PasswordDirectory)
   {
      cp ("--backup=numbered", "./passwd", "./shadow", "./group", "./gshadow", "$BackupDirectory/");
   }
}

if ($Restore)
{
   if (chdir $BackupDirectory)
   {
      print ("Restoring to the $RestoreDirectory from $BackupDirectory\n");
      
      if ($Restore == -1)  #restore the default password files
      {
         print ("Restoring the default files\n");
         cp ("-af", "./passwd", "./shadow", "./group", "./gshadow", "$RestoreDirectory/");
      }
      else           #restore the backed up password files
      {
         print ("Restoring set $Restore of the backup files\n");
         cp ("-af", "./passwd.~$Restore~", "$RestoreDirectory/passwd");
         cp ("-af", "./shadow.~$Restore~", "$RestoreDirectory/shadow");
         cp ("-af", "./group.~$Restore~", "$RestoreDirectory/group");
         cp ("-af", "./gshadow.~$Restore~", "$RestoreDirectory/gshadow");
      }
      
      #because the system will automatically save one set of password files on changes
      #we want to get rid of their versions because our versions probably don't have
      #anything in common with them
      if (chdir $RestoreDirectory)
      {
         print ("Removing the old backup files the system created\n");
         rm ("-f", "./passwd-");
         rm ("-f", "./shadow-");
         rm ("-f", "./group-");
         rm ("-f", "./gshadow-");
      }
   }
}

#if we have too many backups at this point, we need to take care of them
$MAX_TEMP = $MAX_BACKUPS + 1;                    #just a way to make the code simpler later on
if ($Backup && chdir $BackupDirectory && -e "./passwd.~$MAX_TEMP~")
{
   for ($i=1, $j=2; $i <= $MAX_BACKUPS; ++$i, ++$j)
   {
      #rename ("./passd.~j~", "./passwd.~$i~");          #Perl version of doing the same thing as below
      mv ("-f", "./passwd.~$j~", "./passwd.~$i~");
      mv ("-f", "./shadow.~$j~", "./shadow.~$i~");
      mv ("-f", "./group.~$j~", "./group.~$i~");
      mv ("-f", "./gshadow.~$j~", "./gshadow.~$i~");
   }
}
#be all done now
