#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: factory_installation.pl                                           #
#                                                                             #
# Description: Contains the code needed to reinstall the current              #
#              installation.                                                  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

###############################################################################
# GetEmergencyMode
###############################################################################

sub GetEmergencyMode
{
	my $EmergencyMode = '';

	# Try to open the default configuration file...
	if (open(DCC, "< dc.conf"))
	{
		my @ConfigurationData = <DCC>;
		my $Line;

		# Find the preferred language...
		foreach $Line (@ConfigurationData)
		{
			# If the preferred language key is found...
			if ($Line =~ /CID_EMERGENCY_MODE=/)
			{
                # Get the preferred language.
				$Line =~ s/.*CID_EMERGENCY_MODE=(.*)$/$1/;
				chomp($Line);
                
				# Make sure there is something on the key...
				if ((defined($Line)) &&
					($Line ne ''))
				{
					# Use the key value.
					$EmergencyMode = $Line;
				}
				
				last;
			}
		}

		# Close the file handle.
		close(DCC);
	}

	return $EmergencyMode;
}

###############################################################################
# GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $PreferredLanguage = 'en';

	# Try to open the FPIC configuration file...
	if (open(FPICC, "< fpic.conf"))
	{
		my @ConfigurationData = <FPICC>;
		my $Line;

		# Find the preferred language...
		foreach $Line (@ConfigurationData)
		{
			# If the preferred language key is found...
			if ($Line =~ /CID_LANGUAGE=/)
			{
                # Get the preferred language.
				$Line =~ s/.*CID_LANGUAGE=(.*)$/$1/;
				chomp($Line);
                
				# Make sure there is something on the key...
				if ((defined($Line)) &&
					($Line ne ''))
				{
					# Use the key value.
					$PreferredLanguage = $Line;
				}
				
				last;
			}
		}

		# Close the file handle.
		close(FPICC);
	}

	return $PreferredLanguage;
}

###############################################################################
# ResolveResourceFile
###############################################################################

sub ResolveResourceFile
{
	my ($ResourcePath, $ResourceSuffix, $PreferredLanguage) = @_;
	my $Return												= 0;

	# If the preferred resource exists...
	if ( -e $$ResourcePath . '/' . $PreferredLanguage . $ResourceSuffix)
	{
		# Set the resource path.
		$$ResourcePath .= '/'. $PreferredLanguage . $ResourceSuffix;
	}
	elsif ( -e $$ResourcePath . '/en' . $ResourceSuffix)
	{
		# Set the resource path.
		$$ResourcePath .= '/en' . $ResourceSuffix;
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# SetReconstructionScripts
###############################################################################

sub SetReconstructionScripts
{
	my ($ReconstructionScript, $EmergencyRoot, $ProductBinariesRoot)   = @_;
	my $Return 														= 0;
	
	# Open the rc.local on the emergency partition for appendage...
	if (open(RCLOCAL, ">> $EmergencyRoot/etc/rc.d/rc.local"))
	{
		# Append the reconstruction script to the file.
		print RCLOCAL "\n$ProductBinariesRoot/$ReconstructionScript\n";

		# Close the file handle.
		close(RCLOCAL);
	}
	else
	{
		$Return = 1;
	}
	
	return $Return;
}

###############################################################################
# DisableEmergencyDaemon
###############################################################################

sub DisableEmergencyDaemon
{
	my ($Daemon, $EmergencyRoot)	= @_;
	my @RunLevels;
	my $PriorityStart;
	my $PriorityStop;
	my $Return 						= 0;
	
	# Get the start and stop priorities for the daemon...
	if (0 == ($Return = GetDaemonPriorities($Daemon, \@RunLevels, \$PriorityStart, \$PriorityStop)))
	{
		# Disable the stop scripts...
		system("mv -f $EmergencyRoot/etc/rc.d/rc0.d/K$PriorityStop$Daemon $EmergencyRoot/etc/rc.d/rc0.d/disabled.K$PriorityStop$Daemon");
		system("mv -f $EmergencyRoot/etc/rc.d/rc6.d/K$PriorityStop$Daemon $EmergencyRoot/etc/rc.d/rc6.d/disabled.K$PriorityStop$Daemon");
			
		# Disable the start scripts...
		for $i (0 .. $#RunLevels)
		{
			# Disable the start script.
			system("mv -f $EmergencyRoot/etc/rc.d/rc$RunLevels[$i].d/S$PriorityStart$Daemon $EmergencyRoot/etc/rc.d/rc$RunLevels[$i].d/disabled.S$PriorityStart$Daemon");
		}
	}

	return $Return;
}

###############################################################################
# GetDaemonPriorities
###############################################################################

sub GetDaemonPriorities
{
	my ($Script, $RunLevelList, $PriorityStart, $PriorityStop)	= @_;
	my $Return 													= 0;

	# Clear out the values.
	$$PriorityStart = -1;
	$$PriorityStop = -1;

	# Try to open the init script...
	if (open(SCRIPT, "< /etc/rc.d/init.d/$Script"))
	{
		my @ConfigurationData	= <SCRIPT>;
		my $RunLevels			= '';
		my $Line;

		# Run through all of the configuration data.
		foreach $Line (@ConfigurationData)
		{
			# Check for matching line...
			if ($Line =~ /chkconfig:\s*(\d*)\s*(\d*)\s*(\d*)/g)
			{
				# Get the start and stop priorities.
				$RunLevels		= $1;
				$$PriorityStart = $2;
				$$PriorityStop	= $3;

				last;
			}
		}

		# Close the file handle.
		close(SCRIPT);

		# Check for errors...
		if ((-1 == $$PriorityStart) ||
			(-1 == $$PriorityStop))
		{
			$Return = 1;
		}
		elsif ($RunLevels)
		{
			# Run through all of the run levels...
			while ($RunLevels)
			{
				# Push the run level into the list.
				push(@$RunLevelList, chop($RunLevels));
			}
		}
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# SetFDRSplashScreen
###############################################################################

sub SetFDRSplashScreen
{
	my ($SplashScreenPath, $EmergencyRoot, $ProductBinariesRoot)	= @_;
	my $Return 							   							= 0;
	
	# Back up the old splash screen and create the new one.
	system("cp -f $EmergencyRoot/$ProductBinariesRoot/fpsplash.pbm $EmergencyRoot/$ProductBinariesRoot/disabled.fpsplash.pbm");
	system("cp -f $SplashScreenPath $EmergencyRoot/$ProductBinariesRoot/fpsplash.pbm");

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $EmergencyRoot			= '/emergency';
	my $ReconstructionRoot		= '/reconstruction';
	my $ProductBinariesRoot		= '/usr/ins/intel/bin';
	my $ReconstructionScript	= 'reconstruct_images.pl';
	my $EmergencyMode			= GetEmergencyMode();
	my $PreferredLanguage		= GetPreferredLanguage();
	my @FDRPImageSuffixList		= ('fdrp1.pbm', 'fdrp2.pbm', 'fdrp3.pbm');
	my @FDRPImageList;
    my $Return					= 0;
	my $i;

	# Resolve all of the resource file names...
	for $i (0 .. $#FDRPImageSuffixList)
	{
		# Set the product root.
		$FDRPImageList[$i] = $ProductBinariesRoot;

		# Resolve the resource file name...
		if ($Return = ResolveResourceFile(\$FDRPImageList[$i], $FDRPImageSuffixList[$i], $PreferredLanguage))
		{
			last;
		}
	}
	
	# If things are still moving along smoothly...
	if (0 == $Return)
    {
		# Wait a second for the LCD
		# driver to become available.
		sleep(1);

		# If we are not in emergency mode...
		if ($EmergencyMode)
		{
			# Display the processing image on the LCD.
			system("$ProductBinariesRoot/fpil $FDRPImageList[1]");
		}
		else
		{
			# Adjust the reconstruction root.
			$ReconstructionRoot = '/emergency' . $ReconstructionRoot;

			# Display the processing image on the LCD.
			system("$ProductBinariesRoot/fpil $FDRPImageList[0]");
		}

		# Clean out the previous set of data...
        if (0 == ($Return = system("rm -rf $ReconstructionRoot")))
		{
			# Create a clean directory...
			if (mkdir($ReconstructionRoot, 0700))
			{
				# Copy the cleanup image to the emergency partition...
				if (0 == ($Return = system("cp -f $FDRPImageList[2] $ReconstructionRoot/fdrcomplete.pbm")))
				{
					# If we are running in emergency mode...
					if (!$EmergencyMode)
					{
						my $FPICD	= 'fpicd';

						# Initialize the reconstruction scripts, disable the emergency 
						# mode FPIC deamon, and set up the factory default reconstruction 
						# phase one image.
						if ((0 == ($Return = SetReconstructionScripts($ReconstructionScript, $EmergencyRoot, $ProductBinariesRoot))) &&
							(0 == ($Return = DisableEmergencyDaemon($FPICD, $EmergencyRoot))) &&
							(0 == ($Return = SetFDRSplashScreen($FDRPImageList[1], $EmergencyRoot, $ProductBinariesRoot))))
						{
							# Set lilo to boot into the emergency partition 
							# and reboot the system.
							system("/sbin/lilo -R emergency");	
							system("/sbin/shutdown -r 0");
						}
					}
					elsif ($Return = system("$ProductBinariesRoot/$ReconstructionScript"))
					{
						# Adjust the return code.
						$Return /= 256;
					}
				}
				else
				{
					# Adjust the return code.
					$Return /= 256;
				}
			}
			else
			{
				$Return = $!;
			}
		}
		else
		{
			# Adjust the return code.
			$Return /= 256;
		}
	}

	# If there was an error...
	if ($Return)
	{
		# Try to restart the FPIC daemon.
		system("/etc/rc.d/init.d/fpicd start");
	}
    
	return $Return;
}

###############################################################################
# main body
###############################################################################

exit(Main());
