#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: ncu_hosts.pl                                                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

###############################################################################
# GetAddressHash
###############################################################################

sub GetAddressHash
{
	my %AddressHash;
	my @KeyValueSets = split(/,/, shift);
	my $KeyValueSet;
	my $Value;
	my $Key;

	# Set up the hash...
	foreach $KeyValueSet (@KeyValueSets)
	{
		# Break apart the set.
		($Key, $Value) = split(/:/, $KeyValueSet);

		# Set the key/value pair into the hash.
		$AddressHash{$Key} = $Value;
	}

	return %AddressHash;
}

###############################################################################
# AddMapping
###############################################################################

sub AddMapping
{
	my ($HostMap, $AddressNew, $Hostname) = @_;
	my $Domain;

	# Try to open the resolve file...
	if (open(RESOLV, "< /etc/resolv.conf"))
	{
		# Search for the domain...
		while (<RESOLV>)
		{
			# Look for the domain...
			last if ((/domain\s+(\S+)/) && ($Domain = $1));
		}

		# Clean off the line feed.
		chomp($Domain);

		# Close the file.
		close(RESOLV);
	}

	# Build the map (with FQDN if available).
	$$HostMap[0] .= $AddressNew . "\t" . $Hostname;

	# If there is a domain.
	if ($Domain)
	{
		# Add on the regular hostname if FQDN is available.
		$$HostMap[0] .= "\t" . $Hostname . '.' . $Domain;
	}

	# Add on the last line feed.
	$$HostMap[0] .= "\n";
}
							
###############################################################################
# SetMapping
###############################################################################

sub SetMapping
{
	my ($HostMap, $MappedAddress, $AddressNew, $Hostname) = @_;
	my $Line;
	
	# Search through the map...
	foreach $Line (@$HostMap)
	{	
		# If the mapped address is found...
		if ($Line =~ /$MappedAddress/)
		{
			# Swap in the new address and bail out.
			$Line =~ s/$MappedAddress/$AddressNew/;		
		}
	}
}

###############################################################################
# DelMapping
###############################################################################

sub DelMapping
{
	my ($HostMap, $MappedAddress) = @_;
	my $Line;
	
	# Search through the map...
	foreach $Line (@$HostMap)
	{
		# If the mapped address is found...
		if ($Line =~ /$MappedAddress/)
		{
			# Remove the line.
			undef($Line);
			last;
		}
	}
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my %AddressHashNew;
	my $i;
   	
	# Run through the arguments...
	for $i (0 .. $#ARGV)
	{
		# Try to get the new NIC address list...
		if ($ARGV[$i] eq '-nnac')
		{
			# Parse out the data.
			%AddressHashNew = GetAddressHash($ARGV[$i + 1]);
			last;
		}
	}

	# If all of the information is available...
	if (defined(%AddressHashNew))
	{
		my $DelMapping = ((('0.0.0.0' eq $AddressHashNew{'eth0'}) || 
						   (!defined($AddressHashNew{'eth0'}))) && 
						  (('0.0.0.0' eq $AddressHashNew{'eth1'}) ||
						   (!defined($AddressHashNew{'eth1'}))));
		my $Hostname;

		# Try to open the hostname file...
		if (open(HOSTNAME, "< /etc/HOSTNAME"))
		{
			# Get the hostname.
			$Hostname = <HOSTNAME>;

			# Close the file.
			close(HOSTNAME);

			# Get rid of the domain if it exists.
			$Hostname =~ s/(\w*?)\..*/$1/s;
			chomp($Hostname);
			
			# Make sure the node name is not localhost or empty.
			if ((defined($Hostname)) &&
				('localhost' ne $Hostname))
			{
				# Try to open the hosts file...
				if (open(HOSTS, "< /etc/hosts"))
				{
					my $MappedAddress	= '';
					my $AddressNew 		= (($AddressHashNew{'eth0'}) && ('0.0.0.0' ne $AddressHashNew{'eth0'})) ? $AddressHashNew{'eth0'} : $AddressHashNew{'eth1'};
					my @HostMap;
					my $Line;

					# Get the contents of the file.
					@HostMap = <HOSTS>;

					# Close the file.
					close(HOSTS);

					# Run through the file...
					foreach $Line (@HostMap)
					{
						# Try to find the hostname->address mapping in the file...
						last if (($Line =~ /\s$Hostname\s/) && ($MappedAddress = (split(/\t/, $Line))[0]));
					}

					# If there is a modification...
					if ($DelMapping)
					{
						# If the file already has a mapping...
						if ($MappedAddress)
						{
							# Delete the existing mapping.
							DelMapping(\@HostMap, $MappedAddress);
						}
					}
					elsif ($MappedAddress)
					{
						# Update the existing map.
						SetMapping(\@HostMap, $MappedAddress, $AddressNew, $Hostname);
					}
					else
					{
						# Add a new mapping.
						AddMapping(\@HostMap, $AddressNew, $Hostname);
					}

					# Try to open the file for output...
					if (open(HOSTS, "> /etc/hosts"))
					{
						print HOSTS @HostMap;

						# Close the file.
						close(HOSTS);
					}
				}
			}
		}
	}

	return 0;
}

###############################################################################
# main body
###############################################################################

exit(Main());

