#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: ncu_http.pl                                                       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

###############################################################################
# GetAddressHash
###############################################################################

sub GetAddressHash
{
	my %AddressHash;
	my @KeyValueSets = split(/,/, shift);
	my $KeyValueSet;
	my $Value;
	my $Key;

	# Set up the hash...
	foreach $KeyValueSet (@KeyValueSets)
	{
		# Break apart the set.
		($Key, $Value) = split(/:/, $KeyValueSet);

		# Set the key/value pair into the hash.
		$AddressHash{$Key} = $Value;
	}

	return %AddressHash;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $Return = 0;
	my %AddressHashOld;
	my %AddressHashNew;
	my $i;
    
	# Run through the arguments...
	for $i (0 .. $#ARGV)
	{
		# Try to get the old NIC address list...
		if ($ARGV[$i] eq '-onac')
		{
			# Parse out the data.
			%AddressHashOld = GetAddressHash($ARGV[$i + 1]);
		}
		
		# Try to get the new NIC address list...
		if ($ARGV[$i] eq '-nnac')
		{
			# Parse out the data.
			%AddressHashNew = GetAddressHash($ARGV[$i + 1]);
		}

		# Try to bail out early.
		last if ((defined(%AddressHashOld)) && (defined(%AddressHashNew)));
	}

	# If all of the information is available...
	if ((defined(%AddressHashOld)) &&
		(defined(%AddressHashNew)))
	{
		# Make sure the addresses aren't equal.
		if (($AddressHashNew{'eth0'} ne $AddressHashNew{'eth1'}) ||
			(('0.0.0.0' eq $AddressHashNew{'eth0'}) && 
			 ('0.0.0.0' eq $AddressHashNew{'eth1'})))
		{
			# Try to open the hostname file...
			if (open(HOSTNAME, "< /etc/HOSTNAME"))
			{
				my $Domain;

				# Get the hostname.
				$Hostname = <HOSTNAME>;

				# Close the file.
				close(HOSTNAME);

				# Get rid of the domain if it exists.
				$Hostname =~ s/(\w*?)\..*/$1/s;
				chomp($Hostname);

				# Try to open the resolve file...
				if (open(RESOLV, "< /etc/resolv.conf"))
				{
					# Search for the domain...
					while (<RESOLV>)
					{
						# Look for the domain...
						last if ((/domain\s+(\S+)/) && ($Domain = $1));
					}

					# Clean off the line feed.
					chomp($Domain);

					# Close the file.
					close(RESOLV);
				}

				# Try to open the httpd configuration file...
				if (open(HTTPD, "< /etc/httpd/conf/httpd.conf"))
				{
					my $AddressNew				= (($AddressHashNew{'eth0'}) && ('0.0.0.0' ne $AddressHashNew{'eth0'})) ? $AddressHashNew{'eth0'} : $AddressHashNew{'eth1'};
					my $EnableNameVirtualHost	= 0;
					my $NameVirtualHost;
					my $Configuration;
					my $Check;
					my $Cache 					= $/;
					my $Key;

					# Read in the data.
					read(HTTPD, $Configuration, (stat(HTTPD))[7]);

					# Close the file.
					close(HTTPD);

					# Add on the domain if it is available...
					$Hostname .= "." . $Domain;

					# Make sure lower case is used.
					$Hostname = lc($Hostname);

					# Undefine the input record separator.
					undef($/);

					# Check to see if the new address is already in use...
					for $Key (keys(%AddressHashNew))
					{
						# Cache the configuration file.
						$Check = $Configuration;

						# Check for the current new address.
						if ($Check =~ s/.*(<VirtualHost.*?$AddressHashNew{$Key}.*?>.*?<\/VirtualHost>).*/$1/s)
						{
							# Get the current owner of the address.
							$Check =~ s/^.*ServerName\s+?(\S*)\s+?.*$/$1/s;
							$Check = lc($Check);

							# Make sure that the new address maps to a valid configuration.
							if ($Check ne $Hostname)
							{
								# Add the address to the name virtual host list.
								$NameVirtualHost = $AddressHashNew{$Key};

								# Bail out.
								last;
							}
						}
					}

					# Run through the old address list...
					for $Key (keys(%AddressHashOld))
					{

                                               # Replace all instances of the address.
                                               $Configuration =~ s/$AddressHashOld{$Key}/$AddressNew/sg;

                                               # The virtual domain was found.
                                               $EnableNameVirtualHost = 1;

					}

					# If the address already has a representative...
					if ((!$EnableNameVirtualHost) ||
						($Configuration =~ /NameVirtualHost\s+?$NameVirtualHost/s))
					{
						# Remove the address from the array.
						undef($NameVirtualHost);
					}

					# Put the record separator back.
					$/ = $Cache;

					# To to open the file to write to it.
					if (open(HTTPD, "> /etc/httpd/conf/httpd.conf"))
					{
						# Rewrite the configuration.
						print HTTPD $Configuration;

						# Create the remaining name virtual host element...
						if ($NameVirtualHost)
						{
							# Append the directive to the configuration file.
							print HTTPD "NameVirtualHost\t$NameVirtualHost\n";
						}

						# Close the file.
						close(HTTPD);

						# Restart the service.
						`/etc/rc.d/init.d/httpd restart`;
					}
				}                                

			}
		}
		else
		{
			$Return = 1;
		}
	}

	return $Return;
}

###############################################################################
# main body
###############################################################################

exit(Main());

