#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: pdisk_detector.pl                                                 #
#                                                                             #
# Description: Contains the code needed detect a new drive when added to the  #
#              system.                                                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

###############################################################################
# FindPartition
###############################################################################

sub FindPartition 
{
	my ($MountPoint) = @_;
	my $Return = '';
	
	# Get the file system table...
	if (open(FSTAB, "/etc/fstab"))
	{
		my @FileSystemTable = <FSTAB>;
        
		# Close the file.
		close(FSTAB);
		
		# Look through the file...
		foreach $_ (@FileSystemTable) 
		{
			# See if there is a mountpoint match...
			if (/$MountPoint\s/) 
			{
				# Set the return value.
				($Return) = /(\S*)/ ;
				
				last;
			}
		}
   }
   
   return $Return;
}

###############################################################################
# main block
###############################################################################

my $SourceDevice		= '';
my $DestinationDevice	= '';
my $Buffer0				= '';
my $Buffer1				= '';

# Try to find the source device...
if ($SourceDevice = FindPartition("/"))
{
	# Peel off the partition ID.
	chop($SourceDevice);
	
	# Cache the source device.
	$Buffer0 = $SourceDevice;
	
	# Look for a drive match...
	while ($Buffer1 ne 'z')
	{ 
		# Tear off the last character.
		$Buffer1 = chop($Buffer0);
		
		# Bump the character up one.
		$Buffer1++;
		
		# Reappend the new character.
		$Buffer0 = $Buffer0 . $Buffer1;
		
		# Check to see if the device is available...
		if (open(JUNKFH, $Buffer0)) 
		{
			# Close the device.
			close(JUNKFH);
		 
			# Set the destination device.
			$DestinationDevice = $Buffer0;
	
			last;
		}
	}
}

# If there is a destination device.
if ($DestinationDevice)
{
    # Get the drive sizes.
	my $SourceSize		= `/sbin/sfdisk -s $SourceDevice`;
	my $DestinationSize = `/sbin/sfdisk -s $DestinationDevice`;

	# Check to see if the auxiliary disk is too small...
	if ($SourceSize <= $DestinationSize)
	{
		# Send back a 'success' indicator.
		print("Auxiliary drive found.\n");
	}
	else
	{
		# The new drive is too small.
		print("The auxiliary drive is too small.\n");
	}
}
else
{
	# Send back an error indicating that the disk was not found.
	print("Auxiliary drive not available.\n");
}

