#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: raidextend.pl                                                     #
#                                                                             #
# Description: Script to partition /dev/hdb and create a linear raid with it  #
#              and the partition on /dev/hda that mounts on /home or / if     #
#              /home is not listed in /etc/fstab.                             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;

# Global variables to store what devices are used 
my $srcdev="";
my $destdev="";

# determine what devices by the command line
if ($#ARGV == 1) {
   $srcdev = $ARGV[0];
   $destdev = $ARGV[1];
}
# determine what devices by what is mounted on / and the next drive
else{
   $srcdev = findpartition("/");
   chop($srcdev);
   my $temp = $srcdev;
   my $ch = "";
   while ((!$destdev) && ($ch ne "z")) { 
      $ch = chop($temp);
      $ch++;
      $temp = $temp.$ch;
      if (open(JUNKFH, $temp)) {
         close(JUNKFH);
         $destdev = $temp;
      }
   }
   if (!$destdev) {
      die "Unable to determine new device!\n";
   }
}

my $user = `whoami`;
if (!$user =~ "root") {
   die "Must be run as root!\n";
}
open (LOGFILE, ">/var/log/raid0convert.log") || warn "Unable to open /var/log/raid0convert.log\n";
print LOGFILE  "Beginning conversion to linear RAID for /home\n",
               "This logs each step in the process so it can be completed if interrupted\n",
               "or a fixable failure has occurred\n\n";
my $homepart;

# This is the block that does the work.  It logs each step although restart isn't there yet
# If any step fail, it drops out of the loop and logs the error message;
eval  { 
   print LOGFILE  "000, Finding partition mounted on /home...";
   $homepart = findpartition("/home"); 
   if ($homepart =~ "Failure") {
      die "Unable to find partition to extend\n";
   }
   print LOGFILE "found $homepart\n";
   print LOGFILE  "001, Create partition on $destdev\n"; 
   makepartition($destdev);
   # sfdisk used by the makepartition routine doesn't like to set 
   # the partition type to fd when it's created
   print LOGFILE "001a, changing partition type on $destdev.1\n";
   changetype($destdev."1");
   print LOGFILE "001b, removing locks on /home...\n";
   `/etc/rc.d/init.d/dmnASMCheck stop`;
   `/etc/rc.d/init.d/httpd stop`;
   `/etc/rc.d/init.d/identd stop`;
   `killall -9 in.telnetd`;
   sleep(5);
   print LOGFILE "002, unmount current home\n";
   open (TMP, "umount /home 2>&1 |");
   my @tmp = <TMP>;
   close(TMP);
   if (!@tmp =~ "not mounted" && $? != 0) {
      die "unmount failed: @tmp\n";
   }
   print LOGFILE "003, changing partition type on $homepart\n";
   changetype($homepart);
   print LOGFILE "004, creating /etc/raidtab\n";
   createraidtab($homepart,$destdev."1");
   print LOGFILE "005, creating RAID device /dev/md0\n";
   open (TMP, "mkraid --really-force /dev/md0 2>&1 |");
   @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed to create RAID device\n";
   }
   print LOGFILE "006, formatting new RAID device\n";
   open (TMP, "mke2fs /dev/md0 2>&1 |");
   @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed format RAID device\n";
   }
   print LOGFILE "007, mounting new /home\n";
   open (TMP, "mount /dev/md0 /home 2>&1 |");
   @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed to mount new /home\n";
   }
   print LOGFILE "008, editing /etc/fstab to automount /dev/md0 in /home\n";
   editfstab($homepart);
}; 
if ($@) {
   print LOGFILE $@;
   exit(1);
}

`shutdown -r now`;
exit(0);


sub createraidtab {
    my ($part1, $part2) = @_;
    open(RTF, ">/etc/raidtab") || die "Unable to open /etc/raidtab \n";
    print RTF  "raiddev /dev/md0 \n",
               "\traid-level \t linear \n",
               "\tnr-raid-disks \t 2 \n",
               "\tchunk-size \t 32 \n",
               "\tpersistent-superblock \t 1 \n",
               "\tdevice \t ", $part1, "\n",
               "\traid-disk \t 0 \n",
               "\tdevice \t ", $part2, "\n",
               "\traid-disk \t 1 \n";
   close (RTF);
}

sub editfstab {
   my ($part1) = @_;
   open(FSTF, "/etc/fstab") || die "Unable to open /etc/fstab \n";
   my(@whole_thing) = <FSTF>;
   close(FSTF);
   foreach $_  (@whole_thing) {
      s/$part1/\/dev\/md0/;  #replace the partition /dev/hda7 for example with /dev/md0
   }
   open(FSTF, ">/etc/fstab") || die "Unable to open /etc/fstab for writing \n";
   print FSTF @whole_thing;
   close(FSTF);

}

sub makepartition {
   my ($device) = @_;
   use IO::File;
   undef $/;
   my $sfdisk = 'sfdisk -l';
   $device = $destdev unless $device;
   my $fh = new IO::File("$sfdisk 2>/dev/null $device | ");
   unless ( $fh )
   {
	 die "error trying to see device $device\n";
   }
   my $result = <$fh>;
   die "No Device" unless $result;
   $fh->close;
   my $cylinders = $result;
   $cylinders =~ s/.*Disk \S+ (\d+) cylinders, .*/$1/s;
   
$sfdisk = <<EOSFDCMD;
sfdisk 2>/dev/null $device << EOC
0,$cylinders
0,0
0,0
0,0
;
;
EOC
EOSFDCMD
   # Alert:  Note that above the first column is the starting cylinder, the next
   # Alert:  is the size in cylinders, and the third column is default 83
   # Alert:  Apparently this stuff doesn't delete.  It just gets rewritten.
   $fh = new IO::File("$sfdisk | ");
   unless ( $fh )
   {
      print STDERR "error trying to partition $device\n";
      exit(0);
   }
   $result = <$fh>;
   die "Change Rejected" unless $result;
   $fh->close;
   $fh = new IO::File(">>logfile");
   print $fh "re-party-shun result:  \n\n$result";
   $/ = "\n";
}

sub changetype {
   my ($part1) = @_;
   my $device = $part1;
   chop($device);
   my @sfdisk = `/sbin/sfdisk 2>/dev/null -d $device`;
   foreach $_ (@sfdisk) {
      if(/$part1/ ){
         s/Id=83/Id=fd/;
      }
   }
   print @sfdisk;
   open(SFDISK, "| /sbin/sfdisk 2>&1 --no-reread -q $device") || die "unable to execute sfdisk to set type";
   print SFDISK @sfdisk;
   close(SFDISK);
}

sub findpartition {
   my ($mpoint) = @_;
   my $found;
   open (FSTAB, "/etc/fstab") || die "unable to open /etc/fstab for reading";
   my @fstab = <FSTAB>;
   close (FSTAB);
   foreach $_ (@fstab) {
      if (/$mpoint\s/) {
         ($found) = /(\S*)/ ;
      }
   }
   if (!$found) {
      return "Failure to find $mpoint\n";
   }
   return $found;

}
