#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: raidmirror.pl                                                     #
#                                                                             #
# Description: Script to mirror the existing file structure onto the second   #
#              drive.                                                         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;
use Cwd;

# Global variables to store what devices are used 
my $srcdev="";
my $destdev="";

# determine what devices by the command line
if ($#ARGV == 1) {
   $srcdev = $ARGV[0];
   $destdev = $ARGV[1];
}
# determine what devices by what is mounted on / and the next drive
else{
   $srcdev = findpartition("/");
   chop($srcdev);
   my $temp = $srcdev;
   my $ch = "";
   while ((!$destdev) && ($ch ne "z")) { 
      $ch = chop($temp);
      $ch++;
      $temp = $temp.$ch;
      if (open(JUNKFH, $temp)) {
         close(JUNKFH);
         $destdev = $temp;
      }
   }
   if (!$destdev) {
      die "Unable to determine new device!\n";
   }
}

my $user = `whoami`;
die "Must be run as root!\n" if (!$user =~ "root");

open (LOGFILE, ">/var/log/raid1convert.log") || warn "Unable to open logfile\n";
print LOGFILE  "Beginning conversion to linear RAID for /home\n",
               "This logs each step in the process so it can be completed if interrupted\n",
               "or a fixable failure has occurred\n\n";
my $bootpart;

# This is the block that does the work.  It logs each step although restart isn't there yet
# If any step fail, it drops out of the loop and logs the error message;
eval  {
	###########################################################################
	
	print LOGFILE "000, Verifying existance of /boot partition.\n";
	
	$bootpart = findpartition("/boot"); 
	
	if ($bootpart =~ "Failure") {
	  #Can't do this without a separate boot
	  die "Unable to identify boot partition\n";
	}
	
	###########################################################################
	
	print LOGFILE "001, Removing old /etc/raidtab file.\n";
	
	`rm /etc/raidtab` if (-e "/etc/raidtab");
	
	###########################################################################
	
	print LOGFILE "002, Creating RAID mount points..\n";
	
	my $mntpath = "/mnt/raid";

	while (! -e $mntpath && ! -d _) 
	{
		if ( -e _ ) 
		{
			$mntpath = $mntpath . "J";
		}
		else 
		{
			mkdir($mntpath,0777);
		}
	}
	
	###########################################################################
	
	print LOGFILE "003, Duplicating the partition tables onto newauxiliarydrive.\n";
	
	CreateAuxiliaryPartitions($srcdev, $destdev);

	###########################################################################
	
	print LOGFILE "004, Reading existing mount points.\n";
	
	my @mountlist = getallmounting($srcdev);
	my $count = 1; # save /dev/md0 for /
	my $raidtemp = "/dev/md";
	
	###########################################################################
	
	print LOGFILE "005, Creating RAID-1 devices.\n";
	
	foreach $_ (@mountlist) 
	{
		# skip /, /boot and swap since /boot can't be put on raid and / needs to be done last
		# to ensure that the edits done in the /etc/raidtab get onto the raid partition
		if ((/\/boot/) || ($_ eq "swap") || ($_ eq "/")) 
		{ 
			next;
		}

		my $raiddev		= $raidtemp.$count;
		my $raidmntpath	= $mntpath."/".$count;
		
		print LOGFILE "$raiddev, ";
		makeraidpartition($_, $raiddev, $raidmntpath);
		$count++;
	}

	my $raidmntpath = $mntpath . "/0";
	print LOGFILE "/dev/md0\n";
	makeraidpartition("/", "/dev/md0", $raidmntpath);
	
	###########################################################################
	
	print LOGFILE "006, Making ramdisk with RAID-1 module.\n";
	
	makeramdisk();
	
	###########################################################################
	
	print LOGFILE "005, Editing /etc/lilo.conf.\n";
	
	editlilo($raidmntpath);
	
	###########################################################################
	
	print LOGFILE "007, Scheduling cleanup.\n";
	
	open(RCLOCAL, ">>$raidmntpath/etc/rc.d/rc.local");
	print RCLOCAL "\n/usr/ins/intel/bin/raidcleanup.pl\n";
	close(RCLOCAL);
	
	###########################################################################
	
	print LOGFILE "007, Copying non-RAID-1 paritions to newauxiliarydrive.\n";

	CopyAuxiliaryPartitions($srcdev, $destdev);
	
	###########################################################################
	
	print LOGFILE "008, Setting up the MBR on auxiliary drive.\n";
	
	CreateAuxiliaryMBR($srcdev, $destdev, $raidmntpath);
}; 

if ($@) {
   print LOGFILE $@;
   exit(1);
}

print LOGFILE "009, Ready for REBOOT.\n";
`/sbin/shutdown -r now`;
exit(0);

sub createraidtab {
    my ($part1, $part2, $raiddev) = @_;
    open(RTF, ">>/etc/raidtab") || die "Unable to open /etc/raidtab \n";
    print RTF  "raiddev $raiddev \n",
               "\traid-level \t 1 \n",
               "\tnr-raid-disks \t 2 \n",
               "\tchunk-size \t 32 \n",
               "\tpersistent-superblock \t 1 \n",
               "\tdevice \t ", $part2, "\n",
               "\traid-disk \t 0 \n",
               "\tdevice \t ", $part1, "\n",
               "\tfailed-disk \t 1 \n";
   close (RTF);
}

sub editfstab {
   my ($olddev, $newdev) = @_;
   open(FSTF, "/etc/fstab") || die "Unable to open /etc/fstab for reading \n";
   my(@whole_thing) = <FSTF>;
   close(FSTF);
   foreach $_  (@whole_thing) {
      s/$olddev/$newdev/;  #replace the partition /dev/hda7 for example with /dev/md2
   }
   open(FSTF, ">/etc/fstab") || die "Unable to open /etc/fstab for writing \n";
   print FSTF @whole_thing;
   close(FSTF);
}

sub changetype {
   my ($part1) = @_;
   my $device = $part1;
   chop($device);
   my @sfdisk = `/sbin/sfdisk 2>/dev/null -d $device`;
   foreach $_ (@sfdisk) {
      if(/$part1/ ){
         s/Id=83/Id=fd/;
      }
   }
   print @sfdisk;
   open(SFDISK, "| /sbin/sfdisk 2>&1 --no-reread -q $device") || die "unable to execute sfdisk to set type";
   print SFDISK @sfdisk;
   close(SFDISK);
}

sub findpartition {
   my ($mpoint) = @_;
   my $found;
   open (FSTAB, "/etc/fstab") || die "unable to open /etc/fstab for reading";
   my @fstab = <FSTAB>;
   close (FSTAB);
   foreach $_ (@fstab) {
      if (/$mpoint\s/) {
         ($found) = /(\S*)/ ;
      }
   }
   if (!$found) {
      return "Failure to find $mpoint\n";
   }
   return $found;
}

sub getallmounting {
   my ($device) = @_;
   my @returnlist;
   my $tpart;
   my $tmpoint;

   open (FSTAB, "/etc/fstab") || die "unable to open /etc/fstab for reading";
   my @fstab = <FSTAB>;
   close (FSTAB);
   foreach $_ (@fstab) {
      if (/$device/) {
         ($tpart, $tmpoint) = /(\S*)\s*(\S*)/ ; # get the first two words which are the partition and mountpoint
         push(@returnlist, $tmpoint);
      }
   }
   return @returnlist;   
}

sub makeramdisk {

   open(VERSION, "/proc/sys/kernel/osrelease") || die "Unable to determine Kernel version\n";
   my $kversion = <VERSION>;
   close(VERSION);
   chomp $kversion;
   `/sbin/mkinitrd --with=raid1 /boot/raid1-ramdisk $kversion`;
   if ($? != 0) {
      die "mkinitrd returned $?\n";
   }
}

# this sub will edit the lilo.conf that will be mounted once raid comes up
sub editlilo {
   my ($newpath) = @_;
   open(LILO, "$newpath/etc/lilo.conf") || die "Unable to read $newpath/etc/lilo.conf\n";
   my @lilo = <LILO>;
   close(LILO);
   foreach $_ (@lilo) {
      s/root=.*5/root=\/dev\/md0/;
	  s/root=.*8/root=\/dev\/md1/;
	  if ($_ =~ /image=/)
	  {
		$_ .= "\tinitrd=/boot/raid1-ramdisk\n";
	  }
	  elsif (/.*initrd=.*/)
	  {
		s/(.*initrd=.*)/#$1/;
	  }
   }
   open(LILO, ">$newpath/etc/lilo.conf") || die "Unable to write $newpath/etc/lilo.conf\n";
   print LILO @lilo;
   close(LILO);
   my $empart = $srcdev . '3';
   `mkdir /emboot`;
   `/bin/mount $empart /emboot`;
   `/sbin/lilo -C $newpath/etc/lilo.conf`; 
   `/bin/umount /emboot`;
   `rmdir /emboot`;
}

sub makeraidpartition {
   my ($mntpoint, $raiddev, $raidmntpath ) = @_;
   my $srcpart = findpartition($mntpoint);
   my $destpart = $srcpart;
   $destpart =~ s/$srcdev/$destdev/;
   createraidtab($srcpart, $destpart, $raiddev);
   changetype($srcpart);
   changetype($destpart);
   open (TMP, "/sbin/mkraid --really-force $raiddev 2>&1 |");
   my @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed to create RAID device $raiddev\n";
   }
   open (TMP, "/sbin/mke2fs $raiddev 2>&1 |");
   @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed format RAID device $raiddev\n";
   }
   mkdir($raidmntpath,0777); # if this fails it will be caught in the next step
   open (TMP, "/bin/mount $raiddev $raidmntpath 2>&1 |");
   @tmp = <TMP>;
   close (TMP);
   if ($? != 0) {
      print LOGFILE @tmp;
      die "Failed to mount $raiddev to $raidmntpath\n";
   }
   editfstab($srcpart, $raiddev);
   my $mycwd = getcwd;
   chdir $mntpoint;
   `/usr/bin/find . -xdev | /bin/cpio -pm $raidmntpath`;
   chdir $mycwd;
}

###############################################################################
# CopyAuxiliaryPartitions
###############################################################################

sub CopyAuxiliaryPartitions
{
	my ($DeviceSource, $DeviceTarget) 	= @_;
	my @PartitionIDs					= ("1", "3", "4");
	my $PartitionID;
	my $PartitionSource;
	my $PartitionTarget;
	
	# Copy the partitions that don't get put onto RAID...
	foreach $PartitionID (@PartitionIDs) 
	{
		# Build the source and target partitions.
		$PartitionSource = $DeviceSource . $PartitionID;
		$PartitionTarget = $DeviceTarget . $PartitionID;

		# Copy the partition.
		`/bin/dd if=$PartitionSource of=$PartitionTarget`;
	}
}

###############################################################################
# CreateAuxiliaryPartitions
###############################################################################

sub CreateAuxiliaryPartitions
{
	my ($DeviceSource, $DeviceTarget) 	= @_;
	my $DeviceIDSource					= substr($DeviceSource, length($DeviceSource) - 3);
	my $DeviceIDTarget					= substr($DeviceTarget, length($DeviceTarget) - 3);
    my @Partitions			 			= `/sbin/sfdisk 2>/dev/null -uS -d $DeviceSource`;
	
	# Check the results of the partition information read...
	die "Error reading partition table from $DeviceSource" if ($? != 0);
	
	# Wipe the target MBR clean.
	`dd if=/dev/zero of=$DeviceTarget bs=512 count=1`;

	# If this is an IDE source...
	if ($DeviceIDSource =~ /hd/)
	{
		# Update the auxiliary disk geometry.
		UpdateAuxiliaryGeometry($DeviceIDSource, $DeviceIDTarget);
	}

	# Change all of the source devices to targets...
	foreach $_ (@Partitions) 
	{
		# Change the source to the target.
		s/$DeviceSource/$DeviceTarget/ ;
	}

	# Set the partition information to the destination disk...
	open(SFDISK, "| /sbin/sfdisk 2>&1 --no-reread -uS -q -f $DeviceTarget") || die "Error creating partition table on $DeviceTarget";
	print SFDISK @Partitions;
	close(SFDISK);
}

###############################################################################
# UpdateAuxiliaryGeometry
###############################################################################

sub UpdateAuxiliaryGeometry
{
	my ($DeviceIDSource, $DeviceIDTarget) 	= @_;
	
	# Open the source and target geometry files.
	if ((open(SOURCEGEOMETRY, "< /proc/ide/$DeviceIDSource/geometry")) &&
		(open(TARGETGEOMETRY, "< /proc/ide/$DeviceIDTarget/geometry")))
	{
		my @GeometryDataSource	= <SOURCEGEOMETRY>;
		my @GeometryDataTarget	= <TARGETGEOMETRY>;
		my $GeometrySource		= $GeometryDataSource[1];
		my $GeometryTarget		= $GeometryDataTarget[1];
		my $CylindersSource;
		my $CylindersTarget;
		my $CylindersTargetTotal;
		my $HeadsSource;
		my $HeadsTarget;
		my $SectorsSource;
		my $SectorsTarget;

        # Close the file handles.
		close(TARGETGEOMETRY);
		close(SOURCEGEOMETRY);

		# Clean up the end of the lines.
		chomp($GeometrySource);
		chomp($GeometryTarget);

		# Get just the geometry information.
		$GeometrySource =~ s/.*?(\d*\/\d*\/\d*).*/$1/;
		$GeometryTarget =~ s/.*?(\d*\/\d*\/\d*).*/$1/;

		# Break out the individual values.
		($CylindersSource, $HeadsSource, $SectorsSource) = split(/\//, $GeometrySource);  
		($CylindersTarget, $HeadsTarget, $SectorsTarget) = split(/\//, $GeometryTarget);  

		# See if any geometry modifications need to be made...
		if (($HeadsTarget   ne $HeadsSource  ) ||
			($SectorsTarget ne $SectorsSource))
		{
			my $SettingsNew;

			# Calculate the total number of sectors.
			$CylindersTargetTotal	= $CylindersTarget * $HeadsTarget * $SectorsTarget;

			# Match up the sectors and heads to the source drive.
			$SectorsTarget			= $SectorsSource;
			$HeadsTarget			= $HeadsSource;
            			
			# Get the new sector count and make sure it is rounded down.
			$CylindersTarget		= int($CylindersTargetTotal / ($HeadsTarget * $SectorsTarget));

            # Spill up the new settings.
			$SettingsNew = "bios_cyl:$CylindersTarget bios_head:$HeadsTarget bios_sect:$SectorsTarget";

			# Open up the settings file...
			if (open(TARGETSETTINGS, "> /proc/ide/$DeviceIDTarget/settings"))
			{
				# Set the new settings.
				print TARGETSETTINGS $SettingsNew;

				# Close the file.
				close(TARGETSETTINGS);
			}
			else
			{
				die "The new geometry settings ($CylindersTarget/$HeadsTarget/$SectorsTarget) could not be applied to $DeviceIDTarget.";
			}
		}
	}
	else
	{
		# Close the file handle that may be open.
		close(SOURCEGEOMETRY);

		die "Either the source or target device geometry could not be determined.";
	}
}

###############################################################################
# CreateAuxiliaryMBR
###############################################################################

sub CreateAuxiliaryMBR
{
	my ($DeviceSource, $DeviceTarget, $PathExtension)	= @_;
	my $DeviceBoot										= $DeviceTarget . '4';
	my $DeviceEmBoot									= $DeviceTarget . '3';
	my $DeviceRoot										= $DeviceSource . '5';
	my $DeviceEmRoot									= $DeviceSource . '8';
	my $DeviceSwap										= $DeviceTarget . '9';

	# Safe default.
	$PathExtension = '' unless(defined($PathExtension));

	# Read in the first LILO configuraiton file.
	open(LILO, "$PathExtension/etc/lilo.conf") || die "Unable to read $PathExtension/etc/lilo.conf\n";
	my @lilo = <LILO>;
	close(LILO);

	# Massage the device IDs.
	foreach $_ (@lilo) 
	{
		# Set the source device to the destination device.
		s/$srcdev/$destdev/;
	   
		# Patch the root partition locations.
		s/root=\/dev\/md0/root=$DeviceRoot/;
		s/root=\/dev\/md1/root=$DeviceEmRoot/;

		# Replace the boot and emboot Partitions.
		s/\/boot/\/boot-aux/;
		s/\/emboot/\/emboot-aux/;
	}
	
	# Write out the auxiliary LILO configuration file.
	open(LILO, ">$PathExtension/etc/lilo-aux.conf") || die "Unable to write $PathExtension/etc/lilo-aux.conf\n";
	print LILO "disk=$DeviceTarget\n";
	print LILO "\tbios=0x80\n";
	print LILO @lilo;
	close(LILO);

	# Mount the new boot partition.
	`mkdir /boot-aux`;
	`/bin/mount $DeviceBoot /boot-aux`;

	# Mount the new emboot partition.
	`mkdir /emboot-aux`;
	`/bin/mount $DeviceEmBoot /emboot-aux`;
	
	# Set up the auxiliary MBR.
	`/sbin/lilo -C $PathExtension/etc/lilo-aux.conf`; 
	
	# Unmount the new emboot partition.
	`/bin/umount /emboot-aux`;
	`rmdir /emboot-aux`;
	
	# Unmount the new boot partition.
	`/bin/umount /boot-aux`;
	`rmdir /boot-aux`;
	
	# Format the swap disk.
	`/sbin/mkswap $DeviceSwap`;

	# Clean up the mess.
	`rm $PathExtension/etc/lilo-aux.conf`; 
}
