#!/usr/bin/perl 

###############################################################################
#                                                                             #
# Filename: raidrecover.pl                                                    #
#                                                                             #
# Description: Script to recover a failed raid1 mirrored drive.               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;
use Cwd;

# Global variables to store what devices are used 
# determine what devices by the command line
# determine what devices by what is mounted on / and the next drive

my $User = `whoami`;

die "Must be run as root!\n" if (!$User =~ "root");

# Check for the raidcleanup script in rc.local...
if (open(RCLOCAL, "/etc/rc.d/rc.local"))
{
	# Run through the file...
	while (<RCLOCAL>)
	{
		# Look for the raidrecover script...
		if ($_ =~ /raidcleanup/)
		{
			# Nothing to do.
			exit(0);
		}
	}

	# Close the file.
	close(RCLOCAL);
}

# Look in the mdstat file to see if we are running raid1;
open(MDSTAT, "/proc/mdstat");
my @mdstat = <MDSTAT>;
close(MDSTAT);

if (!($mdstat[0] =~ /raid1/) ) 
{
	exit(0); #nothing to do;
}

if (($mdstat[2] =~ /\[UU]/) && 
	($mdstat[3] =~ /\[UU]/) &&
	($mdstat[4] =~ /\[UU]/) &&
	($mdstat[5] =~ /\[UU]/) ||
	(($mdstat[2] =~ /%/) ||
	 ($mdstat[3] =~ /%/) ||
	 ($mdstat[4] =~ /%/) ||
	 ($mdstat[5] =~ /%/))) {
	exit(0); #still nothing to do
}

# Generate alert through ASM for bad drive.
my $sensor = "L_swRAID";
my $type = 2;

system("/usr/ins/intel/bin/asm/bakEmailAlert ERA $sensor $type > /dev/null");
system("/usr/ins/intel/bin/asm/sysWriteToSEL $sensor $type > /dev/null");
system("/usr/ins/intel/bin/asm/sysWriteToLCD $sensor $type > /dev/null");

# Begin the fix process.  Search raidtab for devices to be re-sync'd.

my $i=0;
undef $/;
open (RAIDTAB, "/etc/raidtab");
my $rt = <RAIDTAB>;
close(RAIDTAB);

my @raidtab = split(/raiddev/i,$rt);
my @mdinfo;
my $srcdev;
my $destdev;

foreach $_ (@raidtab) 
{
	if ( /(\/dev\/md\d).*device\s+(\S+).*device\s+(\S+)/s ) 
	{
		if ($2 =~ /a/ ) { #ie hda, the one we are booting on...
			push (@mdinfo, "$1 $3");
			$srcdev = $2;
			$destdev = $3;
		}
		else {
			push (@mdinfo, "$1 $2");
			$srcdev = $3;
			$destdev = $2;
		}
	}
}	

$/ = "\n";
chop($srcdev);
chop($destdev);

# Patch up the new drive.
CreateAuxiliaryPartitions($srcdev, $destdev);
CopyAuxiliaryPartitions($srcdev, $destdev);
CreateAuxiliaryMBR($srcdev, $destdev);

# Call raidhotadd with the failed devices.
for ($i=0; $i < $#mdinfo +1 ; $i++) 
{
	`/sbin/raidhotadd $mdinfo[$i]`;
}

exit(0);

###############################################################################
# CopyAuxiliaryPartitions
###############################################################################

sub CopyAuxiliaryPartitions
{
	my ($DeviceSource, $DeviceTarget) 	= @_;
	my @PartitionIDs					= ("1", "3", "4");
	my $PartitionID;
	my $PartitionSource;
	my $PartitionTarget;
	
	# Copy the partitions that don't get put onto RAID...
	foreach $PartitionID (@PartitionIDs) 
	{
		# Build the source and target partitions.
		$PartitionSource = $DeviceSource . $PartitionID;
		$PartitionTarget = $DeviceTarget . $PartitionID;

		# Copy the partition.
		`/bin/dd if=$PartitionSource of=$PartitionTarget`;
	}
}

###############################################################################
# CreateAuxiliaryPartitions
###############################################################################

sub CreateAuxiliaryPartitions
{
	my ($DeviceSource, $DeviceTarget) 	= @_;
	my $DeviceIDSource					= substr($DeviceSource, length($DeviceSource) - 3);
	my $DeviceIDTarget					= substr($DeviceTarget, length($DeviceTarget) - 3);
    my @Partitions			 			= `/sbin/sfdisk 2>/dev/null -uS -d $DeviceSource`;
	
	# Check the results of the partition information read...
	die "Error reading partition table from $DeviceSource" if ($? != 0);
	
	# Wipe the target MBR clean.
	`dd if=/dev/zero of=$DeviceTarget bs=512 count=1`;

	# If this is an IDE source...
	if ($DeviceIDSource =~ /hd/)
	{
		# Update the auxiliary disk geometry.
		UpdateAuxiliaryGeometry($DeviceIDSource, $DeviceIDTarget);
	}

	# Change all of the source devices to targets...
	foreach $_ (@Partitions) 
	{
		# Change the source to the target.
		s/$DeviceSource/$DeviceTarget/ ;
	}

	# Set the partition information to the destination disk...
	open(SFDISK, "| /sbin/sfdisk 2>&1 --no-reread -uS -q -f $DeviceTarget") || die "Error creating partition table on $DeviceTarget";
	print SFDISK @Partitions;
	close(SFDISK);
}

###############################################################################
# UpdateAuxiliaryGeometry
###############################################################################

sub UpdateAuxiliaryGeometry
{
	my ($DeviceIDSource, $DeviceIDTarget) 	= @_;
	
	# Open the source and target geometry files.
	if ((open(SOURCEGEOMETRY, "< /proc/ide/$DeviceIDSource/geometry")) &&
		(open(TARGETGEOMETRY, "< /proc/ide/$DeviceIDTarget/geometry")))
	{
		my @GeometryDataSource	= <SOURCEGEOMETRY>;
		my @GeometryDataTarget	= <TARGETGEOMETRY>;
		my $GeometrySource		= $GeometryDataSource[1];
		my $GeometryTarget		= $GeometryDataTarget[1];
		my $CylindersSource;
		my $CylindersTarget;
		my $CylindersTargetTotal;
		my $HeadsSource;
		my $HeadsTarget;
		my $SectorsSource;
		my $SectorsTarget;

        # Close the file handles.
		close(TARGETGEOMETRY);
		close(SOURCEGEOMETRY);

		# Clean up the end of the lines.
		chomp($GeometrySource);
		chomp($GeometryTarget);

		# Get just the geometry information.
		$GeometrySource =~ s/.*?(\d*\/\d*\/\d*).*/$1/;
		$GeometryTarget =~ s/.*?(\d*\/\d*\/\d*).*/$1/;

		# Break out the individual values.
		($CylindersSource, $HeadsSource, $SectorsSource) = split(/\//, $GeometrySource);  
		($CylindersTarget, $HeadsTarget, $SectorsTarget) = split(/\//, $GeometryTarget);  

		# See if any geometry modifications need to be made...
		if (($HeadsTarget   ne $HeadsSource  ) ||
			($SectorsTarget ne $SectorsSource))
		{
			my $SettingsNew;

			# Calculate the total number of sectors.
			$CylindersTargetTotal	= $CylindersTarget * $HeadsTarget * $SectorsTarget;

			# Match up the sectors and heads to the source drive.
			$SectorsTarget			= $SectorsSource;
			$HeadsTarget			= $HeadsSource;
            			
			# Get the new sector count and make sure it is rounded down.
			$CylindersTarget		= int($CylindersTargetTotal / ($HeadsTarget * $SectorsTarget));

            # Spill up the new settings.
			$SettingsNew = "bios_cyl:$CylindersTarget bios_head:$HeadsTarget bios_sect:$SectorsTarget";

			# Open up the settings file...
			if (open(TARGETSETTINGS, "> /proc/ide/$DeviceIDTarget/settings"))
			{
				# Set the new settings.
				print TARGETSETTINGS $SettingsNew;

				# Close the file.
				close(TARGETSETTINGS);
			}
			else
			{
				die "The new geometry settings ($CylindersTarget/$HeadsTarget/$SectorsTarget) could not be applied to $DeviceIDTarget.";
			}
		}
	}
	else
	{
		# Close the file handle that may be open.
		close(SOURCEGEOMETRY);

		die "Either the source or target device geometry could not be determined.";
	}
}

###############################################################################
# CreateAuxiliaryMBR
###############################################################################

sub CreateAuxiliaryMBR
{
	my ($DeviceSource, $DeviceTarget, $PathExtension)	= @_;
	my $DeviceBoot										= $DeviceTarget . '4';
	my $DeviceEmBoot									= $DeviceTarget . '3';
	my $DeviceRoot										= $DeviceSource . '5';
	my $DeviceEmRoot									= $DeviceSource . '8';
	my $DeviceSwap										= $DeviceTarget . '9';

	# Safe default.
	$PathExtension = '' unless(defined($PathExtension));

	# Read in the first LILO configuraiton file.
	open(LILO, "$PathExtension/etc/lilo.conf") || die "Unable to read $PathExtension/etc/lilo.conf\n";
	my @lilo = <LILO>;
	close(LILO);

	# Massage the device IDs.
	foreach $_ (@lilo) 
	{
		# Set the source device to the destination device.
		s/$srcdev/$destdev/;
	   
		# Patch the root partition locations.
		s/root=\/dev\/md0/root=$DeviceRoot/;
		s/root=\/dev\/md1/root=$DeviceEmRoot/;

		# Replace the boot and emboot Partitions.
		s/\/boot/\/boot-aux/;
		s/\/emboot/\/emboot-aux/;
	}
	
	# Write out the auxiliary LILO configuration file.
	open(LILO, ">$PathExtension/etc/lilo-aux.conf") || die "Unable to write $PathExtension/etc/lilo-aux.conf\n";
	print LILO "disk=$DeviceTarget\n";
	print LILO "\tbios=0x80\n";
	print LILO @lilo;
	close(LILO);

	# Mount the new boot partition.
	`mkdir /boot-aux`;
	`/bin/mount $DeviceBoot /boot-aux`;

	# Mount the new emboot partition.
	`mkdir /emboot-aux`;
	`/bin/mount $DeviceEmBoot /emboot-aux`;
	
	# Set up the auxiliary MBR.
	`/sbin/lilo -C $PathExtension/etc/lilo-aux.conf`; 
	
	# Unmount the new emboot partition.
	`/bin/umount /emboot-aux`;
	`rmdir /emboot-aux`;
	
	# Unmount the new boot partition.
	`/bin/umount /boot-aux`;
	`rmdir /boot-aux`;
	
	# Format the swap disk.
	`/sbin/mkswap $DeviceSwap`;

	# Clean up the mess.
	`rm $PathExtension/etc/lilo-aux.conf`; 
}
