#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: reconstruct_images.pl                                             #
#                                                                             #
# Description: Contains the code needed to reinstall the current installation.#
#                                                                             #
# WARNING:     This script MUST be run from the /emergency partition and is   #
#              VERY sensitive to drive partition layout.                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

###############################################################################
# RestoreInitializationScripts
###############################################################################

sub RestoreInitializationScripts
{
	my ($ReconstructionScript, $ProductBinariesRoot)   = @_;
	my $Return 				   						= 0;
	my $Cache 										= $/;

	# Undefine the input record separator.
	undef $/;

	# Open the rc.local on the emergency partition for appendage...
	if (open(RCLOCALR, "< /etc/rc.d/rc.local"))
	{
		my $rclocal = <RCLOCALR>;

		# Close the file handle.
		close(RCLOCALR);

		# Remove the reconstruction script to the file.
		$rclocal =~ s/\n$ProductBinariesRoot\/$ReconstructionScript\n//g;
		
		# Open the file again to replace the edited contents.
		if (open(RCLOCALW, "> /etc/rc.d/rc.local"))
		{
			# Spew out the new version of the file.
			print RCLOCALW $rclocal;

			# Close the file handle.
			close(RCLOCALW);
		}
		else
		{
			$Return = 1;
		}
	}
	else
	{
		$Return = 1;
	}
	
	# Put the record separator back.
	$/ = $Cache;

	return $Return;
}

###############################################################################
# EnableDaemon
###############################################################################

sub EnableDaemon
{
	my $Daemon			= shift;
	my @RunLevels;
	my $PriorityStart;
	my $PriorityStop;
	my $Return 			= 0;
	
	# Get the start and stop priorities for the daemon...
	if (0 == ($Return = GetDaemonPriorities($Daemon, \@RunLevels, \$PriorityStart, \$PriorityStop)))
	{
		# Enable the stop scripts...
		system("mv -f /etc/rc.d/rc0.d/disabled.K$PriorityStop$Daemon /etc/rc.d/rc0.d/K$PriorityStop$Daemon");
		system("mv -f /etc/rc.d/rc6.d/disabled.K$PriorityStop$Daemon /etc/rc.d/rc6.d/K$PriorityStop$Daemon");
			
		# Enable the start scripts...
		for $i (0 .. $#RunLevels)
		{
			# Enable the start script.
			system("mv -f /etc/rc.d/rc$RunLevels[$i].d/disabled.S$PriorityStart$Daemon /etc/rc.d/rc$RunLevels[$i].d/S$PriorityStart$Daemon");
		}
	}

	return $Return;
}

###############################################################################
# GetDaemonPriorities
###############################################################################

sub GetDaemonPriorities
{
	my ($Script, $RunLevelList, $PriorityStart, $PriorityStop)	= @_;
	my $Return 													= 0;

	# Clear out the values.
	$$PriorityStart = -1;
	$$PriorityStop	= -1;

	# Try to open the init script...
	if (open(SCRIPT, "< /etc/rc.d/init.d/$Script"))
	{
		my @ConfigurationData	= <SCRIPT>;
		my $RunLevels			= '';
		my $Line;

		# Run through all of the configuration data.
		foreach $Line (@ConfigurationData)
		{
			# Check for matching line...
			if ($Line =~ /chkconfig:\s*(\d*)\s*(\d*)\s*(\d*)/g)
			{
				# Get the start and stop priorities.
				$RunLevels		= $1;
				$$PriorityStart = $2;
				$$PriorityStop	= $3;

				last;
			}
		}

		# Close the file handle.
		close(SCRIPT);

		# Check for errors...
		if ((-1 == $$PriorityStart) ||
			(-1 == $$PriorityStop))
		{
			$Return = 1;
		}
		elsif ($RunLevels)
		{
			# Run through all of the run levels...
			while ($RunLevels)
			{
				# Push the run level into the list.
				push(@$RunLevelList, chop($RunLevels));
			}
		}
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# RestoreSplashScreen
###############################################################################

sub RestoreSplashScreen
{
	my $ProductBinariesRoot	= shift;
	my $Return 			   	= 0;
	
	# Back up the old splash screen and create the new one.
	system("mv -f /$ProductBinariesRoot/disabled.fpsplash.pbm /$ProductBinariesRoot/fpsplash.pbm");

	return $Return;
}

###############################################################################
# FindPartition
###############################################################################

sub FindPartition 
{
	my $MountPoint	= shift;
	my $Return 		= '';
	
	# Get the file system table...
	if (open(FSTAB, "< /etc/fstab"))
	{
		my @FileSystemTable = <FSTAB>;
        
		# Close the file.
		close(FSTAB);
		
		# Look through the file...
		foreach $_ (@FileSystemTable) 
		{
			# See if there is a mountpoint match...
			if (/$MountPoint\s/) 
			{
				# Set the return value.
				($Return) = /(\S*)/ ;
				
				last;
			}
		}
   }
   
   return $Return;
}

###############################################################################
# RestorePartition
###############################################################################

sub RestorePartition
{
	my ($PartitionKey, 
		$PartitionImageMap, 
		$PartitionIDMap, 
		$TargetDevice, 
		$SystemImagesRoot) 		= @_;
	my $bzip2					= '/usr/bin/bzip2';
	my $sfdisk					= '/sbin/sfdisk';
	my $mke2fs					= '/sbin/mke2fs';
	my $e2fsk					= '/sbin/e2fsk';
	my $makedir					= '/bin/mkdir';
	my $mount					= '/bin/mount';
	my $umount					= '/bin/umount';
	my $tar						= '/bin/tar';

	# Reset the partition ID to the EXT2 file system type.
	system("$sfdisk --change-id $TargetDevice $PartitionIDMap->{$PartitionKey} 83");

	# Reset the superblock and group settings.
	system("$mke2fs -F -S $TargetDevice$PartitionIDMap->{$PartitionKey}");

	# Check the partition to make sure it is clean.
	system("$e2fsk -y $TargetDevice$PartitionIDMap->{$PartitionKey}");
	
	# Format the partition to the EXT2 file system type.
	system("$mke2fs -F $TargetDevice$PartitionIDMap->{$PartitionKey}");

	# Make a temporary mount directory.
	system("$makedir $SystemImagesRoot/mnt");
	
	# Special case for root...
	if ($PartitionKey eq 'root')
	{
		# Mount the temporary directory.
		system("$mount $TargetDevice$PartitionIDMap->{$PartitionKey} $SystemImagesRoot/mnt");

		# Make a mount point for /boot.
		system("mkdir -m 0555 $SystemImagesRoot/mnt/boot");
		
		# Make a mount point for /proc.
		system("mkdir -m 0555 $SystemImagesRoot/mnt/proc");
		
		# Make a mount point for /emergency.
		system("mkdir -m 0555 $SystemImagesRoot/mnt/emergency");
		
		# Make a mount point for /var.
		system("mkdir -m 0555 $SystemImagesRoot/mnt/var");
		
		# Make a mount point for /home.
		system("mkdir -m 0555 $SystemImagesRoot/mnt/home");
	}
	else
	{
		# Make a temporary mount directory.
		system("$makedir $SystemImagesRoot/mnt/$PartitionKey");
		
		# Mount the temporary directory.
		system("$mount $TargetDevice$PartitionIDMap->{$PartitionKey} $SystemImagesRoot/mnt/$PartitionKey");
	}
	
    # Uncompress and reset the image.
	system("$tar xI --atime-preserve --same-permissions --same-owner --directory $SystemImagesRoot/mnt --file $SystemImagesRoot/$PartitionImageMap->{$PartitionKey}");

	# Special case for root...
	if ($PartitionKey eq 'root')
	{
		# Unmount the temporary directory.
		system("$umount $SystemImagesRoot/mnt");
	}
	else
	{
		# Unmount the temporary directory.
		system("$umount $SystemImagesRoot/mnt/$PartitionKey");
	}

	# Clean up the temporary mount directory.
	system("rm -rf $SystemImagesRoot/mnt");
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $TargetDevice;
	my $Return = 0;

	# Try to find the source device...
	if ($TargetDevice = FindPartition('/boot'))
	{
		my $SystemImagesRoot		= '/sysimages';
		my $ReconstructionRoot		= '/reconstruction';
		my $ProductBinariesRoot		= '/usr/ins/intel/bin';
		my $ReconstructionScript	= 'reconstruct_images.pl';
		my $bunzip2					= '/usr/bin/bunzip2';
		my $dd						= '/bin/dd';
		my $ImageMBR				= 'mbr.img';
		my %PartitionImageMap		= ( freedos	=>	'freedos.img.bz2', 
										boot	=>	'boot.img.bz2', 
										root	=>	'root.tar.bz2', 
										var		=>	'var.tar.bz2', 
										home	=>	'home.tar.bz2' );
		my %PartitionIDMap			= ( freedos	=>	1, 
										boot	=>	4, 
										root	=>	5, 
										var		=>	6, 
										home	=>	7 );
		my $FPICD					= 'fpicd';
		my $i;
		
		# Peel off the partition ID.
		chop($TargetDevice);
		
		# If the MBR image is available...
		if ( -e ($SystemImagesRoot . '/' . $ImageMBR))
		{
			# Uncompress and reset the MBR image.
			system("$dd if=$SystemImagesRoot/$ImageMBR of=$TargetDevice count=1 bs=512");
		}
	
		# If the FreeDOS image is available...
		if ( -e ($SystemImagesRoot . '/' . $PartitionImageMap{freedos}))
		{
			# Uncompress and reset the FreeDOS image.
			system("$bunzip2 -c $SystemImagesRoot/$PartitionImageMap{freedos} | $dd of=$TargetDevice$PartitionIDMap{freedos}");
		}
		
		# If the /boot image is available...
		if ( -e ($SystemImagesRoot . '/' . $PartitionImageMap{boot}))
		{
			# Uncompress and reset the /boot image.
			system("$bunzip2 -c $SystemImagesRoot/$PartitionImageMap{boot} | $dd of=$TargetDevice$PartitionIDMap{boot}");
		}

		# If the / image is available...
		if ( -e ($SystemImagesRoot . '/' . $PartitionImageMap{root}))
		{
			# Restore the partition.
			RestorePartition('root', \%PartitionImageMap, \%PartitionIDMap, $TargetDevice, $SystemImagesRoot);
		}

		# If the /var image is available...
		if ( -e ($SystemImagesRoot . '/' . $PartitionImageMap{var}))
		{
			# Restore the partition.
			RestorePartition('var',  \%PartitionImageMap, \%PartitionIDMap, $TargetDevice, $SystemImagesRoot);
		}
		
		# If the /home image is available...
		if ( -e ($SystemImagesRoot . '/' . $PartitionImageMap{home}))
		{
			# Restore the partition.
			RestorePartition('home', \%PartitionImageMap, \%PartitionIDMap, $TargetDevice, $SystemImagesRoot);
		}
		
		# Restore the reconstruction scripts, enable the emergency 
		# mode FPIC deamon, and restore the splash screen. 
		if ((0 == ($Return = RestoreInitializationScripts($ReconstructionScript, $ProductBinariesRoot))) &&
			(0 == ($Return = EnableDaemon($FPICD))) &&
			(0 == ($Return = RestoreSplashScreen($ProductBinariesRoot))))
		{
			# Display the processing completed image on the LCD.
			system("$ProductBinariesRoot/fpil $ReconstructionRoot/fdrcomplete.pbm");
	
			# Clean out the cached set of data...
			system("rm -rf $ReconstructionRoot");
			
			# Reboot the system.
			system("/sbin/shutdown -r 0");
		}
	}

	return $Return;
}

###############################################################################
# main body
###############################################################################

exit(Main());
